package org.rsg.interfascia_v2;

import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import org.rsg.interfascia_v2.components.Link;

public abstract class Component {
	protected int font_size = Font.getSize();
	protected String font_face = Font.getFace();
	private int x;
	private int y;
	private int wid;
	private int hgt;
	private String label;
	protected boolean wasClicked;
	protected Object listener;
	protected LookAndFeel lookAndFeel;
	protected Controller controller;
	protected int index;
	public boolean isEnabled = true;
	
	public Component() {
		wasClicked = false;
	}

	public String getFontFace() { return font_face; }
	public void setFontFace(String font_face) { this.font_face = font_face; }
	public int getFontSize() { return font_size; }
	public void setFontSize(int font_size) { this.font_size = font_size; }
	

	public void setIndex(int i) {
		index = i;
	}

	public int getIndex() {
		return index;
	}

	public void update(int i, int j) {
	}

	public void draw() {
	}

	public void draw(int x, int y) {
	}

	public void setController(Controller c) {
		controller = c;
	}

	public Controller getController() {
		return controller;
	}

	public void initWithParent() {
	}

	public void setLookAndFeel(LookAndFeel lf) {
		lookAndFeel = lf;
	}

	public String getLabel() {
		return label;
	}

	public void setLabel(String argLabel) {
		label = argLabel;
	}

	public boolean canReceiveFocus() {
		return true;
	}

	public int getWidth() {
		return wid;
	}

	public void setWidth(int newWidth) {
		if (newWidth > 0)
			wid = newWidth;
	}

	public int getHeight() {
		return hgt;
	}

	public void setHeight(int newHeight) {
		if (newHeight > 0)
			hgt = newHeight;
	}

	public void addActionListener(Object newListener) {
		listener = newListener;
	}

	public void setSize(int newWidth, int newHeight) {
		if (newHeight > 0 && newWidth > 0) {
			hgt = newHeight;
			wid = newWidth;
		}
	}

	public void setPosition(int newX, int newY) {
		if (newX > 0 && newY > 0) {
			x = newX;
			y = newY;
		}
	}

	public void setX(int newX) {
		if (newX > 0)
			x = newX;
	}

	public int getX() {
		return x;
	}

	public void setY(int newY) {
		if (newY > 0)
			y = newY;
	}

	public int getY() {
		return y;
	}

	public void mouseEvent(MouseEvent e) {
		if (e.getID() == java.awt.Event.MOUSE_DOWN) {
			if (isMouseOver(e.getX(), e.getY()))
				wasClicked = true;
		} else if (e.getID() == java.awt.Event.MOUSE_UP && wasClicked
				&& isMouseOver(e.getX(), e.getY())) {
			fireEventNotification(this, "Clicked");
			wasClicked = false;
		}
	}

	public void keyEvent(KeyEvent keyevent) {
	}

	public void actionPerformed(Event guievent) {
	}

	public void fireEventNotification(Component argComponent, String argMessage) {
		if (listener == null)
			return;
		try {
			Event e = new Event(argComponent, argMessage);
			Method m = listener.getClass().getDeclaredMethod("actionPerformed",
					new Class[] { e.getClass() });
			try {
				m.invoke(listener, new Object[] { e });
			} catch (InvocationTargetException ex) {
				System.out.println(ex.getCause().getMessage());
			} catch (IllegalAccessException illegalaccessexception) {
			}
		} catch (NoSuchMethodException ex) {
			System.out.println("NoSuchMethodException");
		}
	}

	public boolean isMouseOver(int mouseX, int mouseY) {
		int w = getWidth();
		int h = getHeight();
		//System.out.println(this + " x:"+x+" y:"+y+" w:"+w+" h:"+h);
		return 	mouseX >= x && 
				mouseY >= y && 
				mouseX <= x + w - 1 && 
				mouseY <= y + h - 1;
	}
}
