package org.rsg.interfascia_v2.components;


import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;

import org.rsg.interfascia_v2.Component;
import org.rsg.interfascia_v2.Font;

import processing.core.PApplet;

public class Button extends Component {
	private final static int FONT_SIZE = 18;

	protected final static int MARGIN_X = 10;	
	protected final static int MARGIN_Y = 3;	
	protected int currentColor;
	protected PApplet papplet; 

	//constructor for fixed width
	public Button(PApplet papplet, String s, Point p, int w) {
		this(papplet, s, FONT_SIZE, p);
	}

	//constructor for width based on label width
	public Button(PApplet papplet, String s, Point p) {
		this(papplet, s, FONT_SIZE, p);
	}

	protected Button(PApplet papplet, String s, int font_size, Point p) {
		this.papplet = papplet; 

		setLabel(s);
		setPosition(p.x, p.y);
		int h = font_size + MARGIN_Y*2;

		Font.setSize(papplet, font_size); 	//have to set font here for textWidth() to work
		int w = (int) papplet.textWidth(s) + MARGIN_X*2;
		Font.setToDefault(papplet);		//then unset it 

		setSize(w, h);
	}


	public void initWithParent() {
		papplet.registerMouseEvent(this);
	}

	public void mouseEvent(MouseEvent e) {
		if (e.getID() == MouseEvent.MOUSE_PRESSED) { //501 == MOUSE_PRESSED
			if (isMouseOver(e.getX(), e.getY()))
				wasClicked = true;
		} else if (	e.getID() == MouseEvent.MOUSE_RELEASED && //502 == MOUSE_RELEASED
					wasClicked && 
					isMouseOver(e.getX(), e.getY())) {
			fireEventNotification(this, "Clicked");
			wasClicked = false;
		}
	}
	
	protected void setColor() {
		//MOUSDOWN ACTIVE
		boolean hasFocus = controller.getFocusStatusForComponent(this);
		if (wasClicked) {
			currentColor = lookAndFeel.activeColor;
			
		//MOUSEOVER HIGHLIGHTING
		} else if (isMouseOver(controller.papplet.mouseX, controller.papplet.mouseY) || hasFocus) {
			currentColor = lookAndFeel.highlightColor;
			
		//NORMAL INACTIVE 
		} else {
			currentColor = lookAndFeel.baseColor;
		}		
	}
	
	protected Rectangle rect() {
		int x = getX();
		int y = getY();
		int h = getHeight();
		int w = getWidth();		
		return new Rectangle(x,y,w,h);
	}
	
	protected void drawBackground(Rectangle r) {
		papplet.stroke(lookAndFeel.borderColor);
		papplet.fill(currentColor);
		papplet.rect(r.x, r.y, r.width, r.height);	
	}

	protected void drawLabel(Rectangle r, int font_size) {
		Font.setSize(papplet,font_size);
		papplet.fill(lookAndFeel.textColor);
		papplet.text(getLabel(), r.x + MARGIN_X -1, r.y + MARGIN_Y +2, r.width, r.height);		
		Font.setToDefault(papplet);		
	}

	public void draw() {
		setColor();
		drawBackground(rect());
		drawLabel(rect(), FONT_SIZE);
	}

	public void keyEvent(KeyEvent e) {
		if (e.getID() == 400 && e.getKeyChar() == ' ')
			fireEventNotification(this, "Selected");
	}

}
