package org.rsg.lib;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.UnsupportedEncodingException;
import java.text.DecimalFormat;
import java.text.FieldPosition;
import java.text.Format;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;

/*
 * some static utilities that do stuff
 */

public class LibUtilities {
//	private final static long startTime = System.currentTimeMillis();
	private static boolean _isWindows = false;
	private static boolean _isLinux = false;
	private static boolean _isMac = false;
	private static boolean _isPPC = false;
	private static boolean _is386 = false;

	static {
	    setOperatingSystems();
	}

	public static String findSuitableFilenameToSave(String prefix, String suffix) {
		int ascii_bottom = 10;
		int ascii_top = 36;
		int loop = 1;
		int i = ascii_bottom;

		while(true) {
			String aa = copyString("" + Character.forDigit(i, 36), loop);
//			System.err.println("[LibUtilities] findSuitableFilenameToSave loop:"+loop+" aa:"+aa);
			String filename = prefix + "_" + dateStamp() + aa + "." + suffix;
			File f = new File(filename);		
			if(!f.exists()) {
//				System.err.println("[LibUtilities] findSuitableFilenameToSave f:"+f.toString());
				return filename;
			}			
			i++;
			
			if(i >= ascii_top) {
				i = ascii_bottom;
				loop++;
			}
		}
	}	

	public final static Format DATE_FORMAT_yyMMMd = new SimpleDateFormat("yyMMMd");
	public static String dateStamp() {
		return DATE_FORMAT_yyMMMd.format(new Date());
	}
	
	public static String copyString(String s, int i) {
		String returnme = "";
		for(int x = 0; x < i; x++) {
			returnme = returnme + s;
		}
		return returnme;
	}
	
	private static void setOperatingSystems() {
		_isWindows = false;
		_isLinux = false;
		_isMac = false;
		_isPPC = false;
		_is386 = false;

		String os 	= System.getProperty("os.name").toLowerCase(Locale.US);
		String arch	= System.getProperty("os.arch").toLowerCase(Locale.US);

		// set the operating system variables
		_isWindows = os.indexOf("windows") != -1;
		_isLinux = os.indexOf("linux") != -1;
		_isMac = os.indexOf("mac") != -1;
		
		//set architecture (for mac)
		_isPPC = arch.indexOf("ppc") != -1;
		_is386 = arch.indexOf("i386") != -1;
    }

	//os
	public static boolean isWindows() { return _isWindows; }
	public static boolean isMac() { return _isMac; }
	public static boolean isLinux() { return _isLinux; }
	
	//arch
	public static boolean isPPC() { return _isPPC; }
	public static boolean is386() { return _is386; }

	//some Mac look-and-feel enhancements -- see Apple Technical Note TN2031
	public static void setOSXSystemProperties(String appName) {
        if (!LibUtilities.isMac()) {
            return;
        }
		System.setProperty("apple.laf.useScreenMenuBar", "true"); //puts menus into finder bar instead of window
		System.setProperty("com.apple.mrj.application.apple.menu.about.name",appName); //sets app name in finder menu
    }
    
	public static void serialize(Object object, String filename) {
		try {
			FileOutputStream fout = new FileOutputStream(filename);
			ObjectOutputStream oos = new ObjectOutputStream(fout);
			oos.writeObject(object);
			oos.close();
		}
		catch (Exception e) { e.printStackTrace(); }
	}

	public static Object unserialize(String filename) {
		Object object = null;
		try {
			FileInputStream fin = new FileInputStream(filename);
			ObjectInputStream ois = new ObjectInputStream(fin);
			object = ois.readObject();
			ois.close();
		}
		catch (Exception e) { e.printStackTrace(); }
		return object;
	} 

    public static String byteToAscii(byte b) {
    	StringBuffer sbuff = new StringBuffer();

    	int i = (b + 256) & 0x00ff;

    	if ((i > 31) && (i < 127)) { 	//only use simple ascii range
    		sbuff.append( (char) i );
    	} else {
    		sbuff.append(" ");		//otherwise print a space
    	}

    	return sbuff.toString();
    }
    
    public static String bytesToAsciiCharString(byte[] data) {
        StringBuffer sbuff = new StringBuffer();
        
        for(int i = 0; i < data.length; i++) {
            //correct for java's lack of unsigned variables
            int b = (data[i] + 256) & 0x00ff;
            
            if ((b > 31) && (b < 127)) { 	//only use simple ascii range
            		sbuff.append( (char) b );
            } else {
        			sbuff.append(" ");		//otherwise print a space
            }
        }
        return sbuff.toString();
    }
    
    public static String bytesToCharString(byte[] data) {
        StringBuffer sbuff = new StringBuffer();
        
        for(int i = 0; i < data.length; i++) {
            //correct for java's lack of unsigned variables
            int b = (data[i] + 256) & 0x00ff;
            
//            if ((b > 31) && (b < 127)) { 	//only use simple ascii range
            		sbuff.append( (char) b );
//            } else {
//        			sbuff.append(" ");		//otherwise print a space
//            }
        }
        return sbuff.toString();
    }

    public static StringBuffer initStringBuffer(int length) {
    	StringBuffer buffer	= new StringBuffer();
    	for(int i = 0; i < length; i++) {
    		buffer.append(" ");
    	}
    	return buffer;
    }

	public static String encodeBytesAsUTF8(byte[] data) {
		 // Decode the bytes into a String
		 String outputString = "";
		 try {
			outputString = new String(data, 0, data.length, "UTF-8");
		 } catch (UnsupportedEncodingException e) {
			e.printStackTrace();
		 }		
		 return outputString;
	}
	
    public static String bytesToFormattedHexAsciiString(byte[] data) {
    	return bytesToFormattedHexAsciiString(data, "");
    }
    public static String bytesToFormattedHexAsciiString(byte[] data, String tabs) {
        int bytes_per_row = 32;
        StringBuffer buffer = new StringBuffer();
		StringBuffer hex = new StringBuffer();
		StringBuffer ascii = new StringBuffer();

        for(int index = 0; index < data.length; index++) {

    		//line wrap
    		if ((index % bytes_per_row) == 0) { 
    			buffer.append(tabs + hex + ascii + "\n"); 
    			hex.delete(0, hex.length());
    			ascii.delete(0, ascii.length());
    		}

        	byte b = data[index];

    		//build hex portion of line
    		int i = (b + 256) & 0x00ff; //correct for java's lack of unsigned variables
    		if (i < 0x10) { hex.append("0"); }
    		hex.append(Integer.toHexString(i));
    		if ((index % 2) == 1) { hex.append(" "); }

    		//build ascii portion of line 
    		ascii.append(byteToAscii(b));
    		
        }
        //add the last bit 
        while(hex.length() < (bytes_per_row * 2.5)) { hex.append(" "); } //pad last line
        buffer.append(tabs + hex + ascii + "\n"); 
        return buffer.toString();
    }

    public static String bytesToHexString(byte[] data) {
        StringBuffer sbuff = new StringBuffer();
        
        for(int i = 0; i < data.length; i++) {
            //correct for java's lack of unsigned variables
            int b = (data[i] + 256) & 0x00ff;
            
            if (b < 0x10)
                sbuff.append("0");
            
            sbuff.append(Integer.toHexString(b));

            if ((i % 2) == 1)
                sbuff.append(" ");
        }
        
        return sbuff.toString();
    }

    public static String formatFloat(float f) {
        DecimalFormat decimalformat = new DecimalFormat("#,##0.00");
        FieldPosition fieldposition = new FieldPosition(0);
        StringBuffer stringbuffer = new StringBuffer();
        decimalformat.format(f, stringbuffer, fieldposition);
        return stringbuffer.toString();
    }
    
	public static File getCurrentPath() {
		File f = null;
		try {
			f = new File(new File(".").getCanonicalPath());
		} catch (IOException e) {
			e.printStackTrace();
		}	
		return f;
	}
	
//	public static long currentTime() {
//		return System.currentTimeMillis() - startTime;
//	}

	//add to PApplet?
	//does the inverse of "constrain" 
	static public final float exclude(float amt, float low, float high){
	  return ((amt < low) || (amt > high)) ? amt : (((amt - low) < (high - amt)) ? low : high);
	}
	
	static public final int exclude(int amt, int low, int high){
	  return ((amt < low) || (amt > high)) ? amt : (((amt - low) < (high - amt)) ? low : high);
	}
}
